/* debug.c   The GPLBASIC interpreter
   Copyright (C) 1997 L.C. Benschop, Eindhoven, the Netherlands
   */

#include "gplbasic.h"
#include "symtab.h"
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>


static void parse_progname(void)
{
  int i=0,j=0;
  while( progline[i]==' ' || progline[i]=='\t' ) i++;
  if(progline[i]==0) return;
  while( progline[i]!=' ' && progline[i]!='\t' && progline[i]!=0)i++;  
  while( progline[i]==' ' || progline[i]=='\t' ) i++;
  if(progline[i]==0) return;
  while( progline[i]!=' ' && progline[i]!='\t' && progline[i]!='\n' ) {
    progname[j++]=progline[i++];
  }
  progname[j]=0;
}

static void dbg_load(void) 
{
  parse_progname();
  load_program(progname);
}

static void dbg_edit(void)
{
  char *p;
  parse_progname();
  if((p=getenv("VISUAL"))==0 && (p=getenv("EDITOR"))==0)p="vi";
  strcpy(progline,p);
  strcat(progline," ");
  strcat(progline,progname);
  system(progline);
  load_program(progname);
}

static void dbg_lvar(void)
{
  list_vars();
}

static void dbg_run(void)
{
  clear_vars();
  run_program(0);
}

static void dbg_bye(void)
{
  exit(0);
}

static void dbg_mem(void)
{
  int n;
  unsigned char *p;
  if(progareas==0)n=0;
  else n=sizeof(CELLTYPE)*((progareas-1)*S_PROGCHUNK+cells_used);
  printf("Used %d bytes of program memroy in %d areas.\n",n,progareas);
  if(charareas==0)n=0;
  else n=(charareas-1)*S_STRCONST+chars_used;
  printf("Used %d bytes of character constant memory in %d areas.\n",n,
  	charareas);
  if(floatareas==0)n=0;
  else n=sizeof(double)*((floatareas-1)*S_FCONST+floats_used);
  printf("Used %d bytes of floating point constant memory in %d areas.\n",
   	n,floatareas);

  if(symareas==0)n=0;
  else n=sizeof(VARSYM)*((symareas-1)*S_SYMCHUNK+syms_used);
  printf("Used %d bytes of symbol table memory in %d areas.\n",n,symareas);
  if(labareas==0)n=0;
  else n=sizeof(LABSYM)*((labareas-1)*S_SYMCHUNK+labs_used);
  printf("Used %d bytes of label table memory in %d areas.\n",n,labareas);
  printf("Used %d memory areas out of %d.\n",memareas,N_MEMAREAS);
  p=stringareas;n=0;
  while(p) {
    n++;
    p=*((char **)p);
  }
  printf("Strings: %d bytes in %d areas.\n",S_STRINGAREA*n,n);
}


typedef struct { /* command table for debugger */
  char *cmdname;
  void (*func)(void);
}DEBUGCMDTYPE;

static DEBUGCMDTYPE debug_commands[]= {
  {"RUN",dbg_run},
  {"BYE",dbg_bye},
  {"QUIT",dbg_bye},
  {"SYSTEM",dbg_bye},
  {"MEM",dbg_mem},
  {"LOAD",dbg_load},
  {"EDIT",dbg_edit},
  {"LVAR",dbg_lvar},
};

static char keyword[10];

static void parse_keyword(void) 
{
  int i=0,j=0;
  char c;
  while( progline[i]==' ' || progline[i]=='\t' ) i++;
  while(isalpha(progline[i]) && j<9) {
    c=progline[i++];if(islower(c)) c=toupper(c);
    keyword[j++]=c;
  }
  keyword[j]=0;
}



void debugger(void) {
  int i;
  for(;;) {
    if(setjmp(errorbuf)){
      printf("Press ENTER to return to prompt\n");
      fgets(progline,MAXLINELENGTH+2,stdin);
    }
    printf("basic debug>");fflush(stdout);
    if(!fgets(progline,MAXLINELENGTH+2,stdin))exit(0);
    if(progline[0]=='\n') continue; /* skip empty line */
    parse_keyword();
    for(i=0;i<sizeof(debug_commands)/sizeof(DEBUGCMDTYPE);i++) {
      if(!strcmp(debug_commands[i].cmdname,keyword)) break;
    }    
    if(i<sizeof(debug_commands)/sizeof(DEBUGCMDTYPE)) { /* command found*/
      (debug_commands[i].func)();
    } else {
      printf("Unknown command\n");
    }
  }
}

