/* main.c   The GPLBASIC interpreter
   Copyright (C) 1997 L.C. Benschop, Eindhoven, the Netherlands
   */

#include "gplbasic.h"
#include "errors.h"
#include "token.h"
#include <stdio.h>
#include <stdlib.h>

typedef struct {
         int number; /* Error message number */
         char * message; /* Error message */
         char flag;   /* various flags */
         char berr,qerr; /* ERR numbers in bbc or qbasic */
} MESGTYPE;

MESGTYPE messages[] = 
{ { GE_OK , "Ok" , 0 },
  { GE_OFR , "Cannot open file \"%s\" for reading" , 1 , 0, 0},
  { GE_OFW , "Cannot open file \"%s\" for writing" , 1 , 0 , 0},
  { GE_OFA , "Cannot open file \"%s\" for appending" , 1 , 0 , 0},
  { GE_OFU , "Cannot open file \"%s\" for random access" , 1, 0 , 0},
  { GE_OFB , "Cannot open file \"%s\" for binary access" , 1 , 0 , 0},
  { GE_STP , "Stop statement", 0, 0 ,0},
  { GE_BRK , "Program interrupted", 0, 0 ,0},
  { GE_DIV , "Division by zero", 0 , 0, 0},
  { GE_FPE , "Floating point error", 0, 0, 0},
  { GE_ARG , "Invalid argument to function %s", 1 , 0 ,0},
  { GE_OVF , "Integer overflow" , 0 , 0 ,0},
  { GE_ARR , "Cannot allocate memory for array %s", 1 , 0 ,0},
  { GE_STR , "Cannot allocate memory for strings", 1 , 0 , 0},
  { GE_SUB , "Invalid subscript for array %s", 1 , 0 , 0},
  { GE_STK , "Stack overflow", 0 , 0 ,0},
  { GE_STS , "String stack overflow", 0 , 0 ,0},
  { GE_STF , "Floating point stack overflow", 0 , 0 , 0},
  { GE_UND , "Stack underflow", 2 , 0 , 0},
  { GE_OPC , "Invalid Basic opcode", 2 , 0 , 0},
  { GE_COR , "String space corrupted", 2 , 0 , 0},
  { GE_SEG , "Segmentation fault", 2 , 0 , 0},
  { GE_EXP , "Expression syntax error", 3 , 0 , 0},
  { GE_SYN , "Syntax error", 3 , 0 , 0},
  { GE_TYP , "Type mismatch", 3 , 0 , 0},
  { GE_WIT , "%s without %s", 5 , 0 , 0},
  { GE_MEM , "Not enough memory to load program", 3 , 0 , 0},
  { GE_LAB , "Undefined label %s", 4 , 0 , 0},
  { GE_FUN , "Undefined function %s", 4 , 0 , 0},
  { GE_SBR , "Undefined subroutine %s", 4 , 0 ,0},
  { GE_PMM , "Parameter mismatch in call to %s", 4 , 0 ,0 },
  { GE_DUP , "Duplicate definition of label %s", 4 , 0 ,0 },
  { GE_DIM , "Multiple DIM of array %s" , 1 , 0 , 0 },
  { GE_DAT , "Out of data", 0 , 0 , 0 },
  { GE_BFN , "Bad file number", 0 , 0 , 0},
};

void doerror(int errnum, char * mes1, char * mes2)
{
 int i,j;
 if(!mes1)mes1="";
 if(!mes2)mes2="";
 for(i=0;i<sizeof(messages)/sizeof(MESGTYPE);i++) {
   if(errnum==messages[i].number) {
     fprintf(stderr,messages[i].message,mes1,mes2);
     if(lineno) { /* occurred during compiling of source program? */
       fprintf(stderr," in line %ld\n%s",lineno,progline);
       for(j=0;j<parseptr-progline;j++)fprintf(stderr," ");
       fprintf(stderr,"^");
     } else if(last_line) { /* at runtime and source line tracking enabled */
       fprintf(stderr," in line %ld",last_line->l);
     }
     fprintf(stderr,"\n");
     longjmp(errorbuf,1);
   }
 }
}


