/* main.c   The GPLBASIC interpreter
   Copyright (C) 1997 L.C. Benschop, Eindhoven, the Netherlands
   */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <string.h>

#include "gplbasic.h"
#include "errors.h"

#ifdef CONFIG_TERM
#include "terminal.h"
#endif
#ifdef CONFIG_X
#include "xgraph.h"
#endif

void * mem_areas[N_MEMAREAS]; /* Allocated memory areas */
unsigned char string_stack[S_STRINGSTACK]; 
double float_stack[S_FLTSTACK];
CELLTYPE main_stack[S_MAINSTACK];

jmp_buf errorbuf;

char progname[257]; /* name of program being run */


void brk_handler(int x)
{
  signal(SIGINT,brk_handler);
  doerror(GE_BRK,0,0);
}

void seg_handler(int x)
{
  signal(SIGSEGV,seg_handler);
  doerror(GE_SEG,0,0);
}

void fpe_handler(int x)
{
  signal(SIGFPE,fpe_handler);
  doerror(GE_FPE,0,0);
}

void exit_basic(int exit_code)
{
#ifdef CONFIG_X
 exit_x();
#endif
#ifdef CONFIG_TERM
 exit_term();
#endif
 exit(exit_code);
}

int main ( int argc , char * argv[] )
{
  int debugging=0;
  int progloaded=0;
#ifdef CONFIG_TERM
  init_term();
#endif
#ifdef CONFIG_X
  init_x();
#endif
  init_keywords(1);
  init_strings();
  errhandling=0;
  if(setjmp(errorbuf)) {
    exit_basic(1);
  }
  signal(SIGINT,brk_handler);
  signal(SIGFPE,fpe_handler);
  signal(SIGSEGV,seg_handler);
  for(;;) {
    if( strcmp( argv[1] , "-d") == 0) {
      argv++;argc--;
      debugging=1;
    } else if( strcmp( argv[1] , "-e") == 0) {
      argv++;argc--;
      errhandling=1;
    } else break;
  } if (argc>=2) {
    argv++;argc--;
    strcpy(progname,argv[0]);
    progloaded=!load_program(progname);
  } 
  if(debugging) { 
    debugger();
  } else if(progloaded) {
    run_program(0);
  } else exit_basic(1);
  exit_basic(0);
}



